/*
 * Copyright 2016 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.urmc;

import java.io.PrintStream;
import java.util.Arrays;

import at.jku.risc.stout.urmc.algo.DebugLevel;
import at.jku.risc.stout.urmc.algo.MatchAlgorithm;
import at.jku.risc.stout.urmc.algo.MatchProblem;
import at.jku.risc.stout.urmc.algo.MatchSystem;
import at.jku.risc.stout.urmc.algo.Substitution;
import at.jku.risc.stout.urmc.data.InputParser;
import at.jku.risc.stout.urmc.data.atom.NodeFactory;

public class MatchMain {

	static DebugLevel debugLevel = DebugLevel.SIMPLE;
	static boolean justifySigma = false;

	public static void main(String[] args) {
		if (args.length >= 1) {
			if ("-d".equals(args[0])) {
				debugLevel = DebugLevel.PROGRESS;
				main(Arrays.copyOfRange(args, 1, args.length));
			} else if ("-h".equals(args[0])) {
				printHelp(System.out);
			} else if ("-j".equals(args[0])) {
				justifySigma = true;
				main(Arrays.copyOfRange(args, 1, args.length));
			} else {
				try {
					MatchProblem p = new MatchProblem();
					new InputParser(p, new NodeFactory()).parseEqSystem(
							args[0], null);
					new MatchAlgorithm(new MatchSystem(p, new Substitution()))
							.match(debugLevel, System.out, System.out,
									justifySigma);

					System.out.println();
					System.out
							.println("Supported by the Austrian Science Fund (FWF) "
									+ "under the project SToUT P 24087-N18");
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		} else {
			System.out.println("Insufficient parameters!");
			System.out.println();
			printHelp(System.out);
		}
	}

	private static void printHelp(PrintStream out) {
		System.out
				.println("Context & sequence matching problem solver for unranked terms");
		System.out.println();
		System.out.println("Give a problem of the form:");
		System.out.println("\"s1 << t1; s2 << t2; ...; sn << tn\"");
		System.out.println("where s1,...,sn and t1,...,tn are terms or hedges");
		System.out.println("Input example: \"C(f(X)) << g(f(a,b),h(f(a),f))\"");
		System.out.println();
		System.out.println("Simple execution example: "
				+ "java at.jku.risc.stout.urmc.MatchMain -a "
				+ "\"f(a,X) << f(a,b,c,d)\"");
		System.out.println("Advanced execution example with jvm parameters:");
		System.out.println("java -Xmx2048m -XX:MaxPermSize=1024m -Xss128m "
				+ "at.jku.risc.stout.urmc.MatchMain -j -d "
				+ "\"f(a,X) << f(a,b,c,d)\"");
		System.out.println();
		System.out.println("  -j    justify computed unifiers");
		System.out.println("  -h    print this information");
		System.out
				.println("  -d    debug mode prints out every available information");
	}
}
